@@ -82,6 +82,21 @@ All auxiliary data can be obtained by prepending the channel name such as
     ``pixel_quality`` and disambiguated by a to-be-decided property in the
     `DataID`.
 
+.. note::
+
+    For reading compressed data, a decompression library is
+    needed. Either install the FCIDECOMP library (see `PUG`_), or the
+    ``hdf5plugin`` package with::
+
+        pip install hdf5plugin
+
+    or::
+
+        conda install hdf5plugin -c conda-forge
+
+    If you use ``hdf5plugin``, make sure to add the line ``import hdf5plugin``
+    at the top of your script.
+
 .. _PUG: https://www-cdn.eumetsat.int/files/2020-07/pdf_mtg_fci_l1_pug.pdf
 .. _EUMETSAT: https://www.eumetsat.int/mtg-flexible-combined-imager  # noqa: E501
 .. _test data release: https://www.eumetsat.int/simulated-mtg-fci-l1c-enhanced-non-nominal-datasets
@@ -307,10 +322,10 @@ class FCIL1cNCFileHandler(NetCDF4FileHandler):
         """Get the auxiliary data arrays using the index map."""
         # get index map
         index_map = self._get_dataset_index_map(_get_channel_name_from_dsname(dsname))
-        # index map indexing starts from 1
-        index_map -= 1
+        # subtract minimum of index variable (index_offset)
+        index_map -= np.min(self['index'])
 
-        # get lut values from 1-d vector
+        # get lut values from 1-d vector variable
         lut = self._get_aux_data_lut_vector(_get_aux_data_name_from_dsname(dsname))
 
         # assign lut values based on index map indices
@@ -349,6 +364,17 @@ class FCIL1cNCFileHandler(NetCDF4FileHandler):
         extents = {}
         for coord in "xy":
             coord_radian = self["data/{:s}/measured/{:s}".format(channel_name, coord)]
+
+            # TODO remove this check when old versions of IDPF test data (<v4) are deprecated.
+            if coord == "x" and coord_radian.scale_factor > 0:
+                coord_radian.attrs['scale_factor'] *= -1
+
+            # TODO remove this check when old versions of IDPF test data (<v5) are deprecated.
+            if type(coord_radian.scale_factor) is np.float32:
+                coord_radian.attrs['scale_factor'] = coord_radian.attrs['scale_factor'].astype('float64')
+            if type(coord_radian.add_offset) is np.float32:
+                coord_radian.attrs['add_offset'] = coord_radian.attrs['add_offset'].astype('float64')
+
             coord_radian_num = coord_radian[:] * coord_radian.scale_factor + coord_radian.add_offset
 
             # FCI defines pixels by centroids (see PUG), while pyresample
@@ -527,5 +553,12 @@ class FCIL1cNCFileHandler(NetCDF4FileHandler):
 
         sun_earth_distance = np.mean(self["state/celestial/earth_sun_distance"]) / 149597870.7  # [AU]
 
+        # TODO remove this check when old versions of IDPF test data (<v5) are deprecated.
+        if sun_earth_distance < 0.9 or sun_earth_distance > 1.1:
+            logger.info('The variable state/celestial/earth_sun_distance contains unexpected values'
+                        '(mean value is {} AU). Defaulting to 1 AU for reflectance calculation.'
+                        ''.format(sun_earth_distance))
+            sun_earth_distance = 1
+
         res = 100 * radiance * np.pi * sun_earth_distance ** 2 / cesi
         return res
